/*
 * Raima Database Manager
 *
 * Copyright (c) 2019 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/*! \file
 * \brief Source code for the RDM core05 example
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "print_error.h"
#include "rdm.h"

/* Generated \c struct and \c typedef definitions to be used with the RDM APIs */
#include "core05_db_structs.h"

/* Generated catalog definition to be used with the RDM rdm_dbSetCatalog() API */
#include "core05_db_cat.h"

/*!
 * \mainpage Core05 Popcorn Example
 *
 * Each time you run this example:
 * \li The database is initialized (if necessary)
 * \li 5 new person records are added to the database
 * \li A list of each person is displayed in index order.
 *
 * \par Table of Contents
 *
 * - \subpage hDB
 * - \subpage hRDMfunc
 * - \subpage hPGMfunc
 *
 * For additional information, please refer to the product documentation at http://docs.raima.com/.
 *
 * \page hDB Database Schema
 *
 * \par Database Schema Definition
 *
 * The DDL (Database Definition Language) specification for the database used
 * in this example is located in the file \c core05_db.sdl.
 *
 * \include core05_db.sdl
 *
 * The schema was compiled using the RDM rdm-compile utility with the -s option to
 * generate C-structures for interfacing with the database.
 *
 * \code rdm-compile -s core05_db.sdl \endcode
 *
 * \page hRDMfunc RDM Functions Used
 * The following RDM Functions are illustrated:
 * \li rdm_rdmAllocDatabase/rdm_dbFree
 * \li rdm_dbSetCatalog
 * \li rdm_dbOpen/rdm_dbClose
 * \li rdm_dbInsertRow
 * \li rdm_dbAllocCursor/rdm_cursorFree
 * \li rdm_dbGetRowsByKey
 * \li rdm_cursorMoveToFirst/rdm_cursorMoveToNext
 * \li rdm_cursorReadRow
 *
 * They are fully documented in http://docs.raima.com/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value sOKAY indicates a successful call.
 *
 * The actual database will be stored in a directory named 'core05.rdm' in the
 * project directory.
 *
 * \page hPGMfunc Program Functions
 * \li main() - \copybrief main
 */

/*! \brief Main function for core05 example
 *
 * The function initializes the RDM environment and runs the create, read operations.
 *
 * @return Returns the \c RDM_RETCODE on exit.
 */
int main (void)
{
    RDM_DB hDB;
    RDM_RETCODE rc;
    PERSON person_rec;

    /* Seed the random number generator */
    srand ((unsigned int) time (NULL));

    /* allocate a database handle using the default TFS */
    rc = rdm_rdmAllocDatabase (&hDB);
    print_error (rc);

    if (rc == sOKAY)
    {
        /* Override the default DURABLE setting to CONSISTENT */
        rc = rdm_dbSetOptions (*pDB, "durability=consistent");
        print_error (rc);

        /* Associate the precompiled catalog with the datbase handle */
        rc = rdm_dbSetCatalog (hDB, core05_db_cat);
        print_error (rc);

        /* open the database */
        /* if the database does not exits, it will be created */
        if (rc == sOKAY)
        {
            rc = rdm_dbOpen (hDB, "core05", RDM_OPEN_EXCLUSIVE);
            print_error (rc);
        }

        /* add 5 new records to the database */
        if (rc == sOKAY)
        {
            int ii;

            printf ("Adding 5 new person records\n");
            for (ii = 0; ii < 5; ii++)
            {
                unsigned int value;

                value = (unsigned int) rand () % 10000;
                sprintf (person_rec.LAST_FOUR_SSN, "%04u", value);
                rc = rdm_dbInsertRow (hDB, TABLE_PERSON, &person_rec, sizeof (person_rec), NULL);
                print_error (rc);
                printf ("\t%s\n", person_rec.LAST_FOUR_SSN);
            }
        }

        /* Loop through and display each person record through the index */
        if (rc == sOKAY)
        {
            RDM_CURSOR cursor = NULL;

            printf ("\nList all records in index order\n");
            rc = rdm_dbGetRowsByKey (hDB, KEY_PERSON_LAST_FOUR_SSN, &cursor);
            print_error (rc);

            if (rc == sOKAY)
            {
                for (rc = rdm_cursorMoveToFirst (cursor); rc == sOKAY; rc = rdm_cursorMoveToNext (cursor))
                {
                    rc = rdm_cursorReadRow (cursor, &person_rec, sizeof (person_rec), NULL);
                    print_error (rc);

                    printf ("%s\n", person_rec.LAST_FOUR_SSN);
                }
                rdm_cursorFree (cursor);
            }
        }

        /* free the database handle */
        rdm_dbFree (hDB);
    }

    return (int) rc;
}
